﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using Devart.Data.PostgreSql;
using Devart.Data.PostgreSql.Entity;
using Devart.Data.PostgreSql.Entity.Configuration;
using System.ComponentModel.DataAnnotations;

namespace PostgreSQLFullTextSearch
{
    class Program
    {
        static void Main(string[] args)
        {
            var monitor = new PgSqlMonitor() { IsActive = true };

            Database.SetInitializer<PostgreSQLFullTextSearch.MyContext>(null);

            using (var ctx = new PostgreSQLFullTextSearch.MyContext())
            {

                foreach (Book b in ctx.Books)
                    ctx.Books.Remove(b);
                ctx.SaveChanges();

                // Load information into the Books table
                ctx.Books.Add(new PostgreSQLFullTextSearch.Book()
                {
                    Author = "William Shakespeare",
                    Name = "The Tragedy of King Richard the Second",
                    Content = @"ACT 1
SCENE I. London. A Room in the palace.
[Enter KING RICHARD, attended; JOHN OF GAUNT, with other NOBLES.]
KING RICHARD.
  Old John of Gaunt, time-honoured Lancaster,
  Hast thou, according to thy oath and band,
  Brought hither Henry Hereford thy bold son,
  Here to make good the boisterous late appeal,
  Which then our leisure would not let us hear,
  Against the Duke of Norfolk, Thomas Mowbray?
..."
                });
                ctx.Books.Add(new PostgreSQLFullTextSearch.Book()
                {
                    Author = "William Shakespeare",
                    Name = "The Life and Death of King John",
                    Content = @"ACT 1.
SCENE I. Northampton. A Room of State in the Palace.
[Enter KING JOHN, QUEEN ELINOR, PEMBROKE, ESSEX, SALISBURY, and others, with CHATILLON.]
KING JOHN.
  Now, say, Chatillon, what would France with us?
CHATILLON.
  Thus, after greeting, speaks the King of France,
  In my behaviour, to the majesty,
  The borrow'd majesty of England here.
..."
                });
                ctx.Books.Add(new PostgreSQLFullTextSearch.Book()
                {
                    Author = "William Shakespeare",
                    Name = "The Life of Henry the Fifth",
                    Content = @"ACT 1.
SCENE I. London. An ante-chamber in the King's palace.
[Enter the Archbishop of Canterbury and the Bishop of Ely.]
CANTERBURY.
  My lord, I'll tell you: that self bill is urg'd,
  Which in the eleventh year of the last king's reign
  Was like, and had indeed against us pass'd,
  But that the scambling and unquiet time
  Did push it out of farther question.
..."
                });
                ctx.SaveChanges();

                // Write an analogue of the SQL query:
                //  SELECT author, name FROM books
                //  WHERE to_tsvector(content) @@ plainto_tsquery('king palace Norfolk')
                var firstQuery = ctx.Books
                    .Where(b => PgSqlTextFunctions.Match(
                        PgSqlTextFunctions.ToTsVector(b.Content), 
                        PgSqlTextFunctions.PlainToTsQuery("king palace Norfolk"))
                    )
                    .Select(b => new { b.Author, b.Name });
                var firstQueryResults = firstQuery.ToList();

                // Output:
                Console.WriteLine();
                Console.WriteLine("== Querying your single column full-text index ==");
                Console.WriteLine();
                Console.WriteLine("1) Natural language");
                Console.WriteLine();
                Console.WriteLine("SQL:");
                Console.WriteLine(firstQuery.ToString());
                Console.WriteLine();
                Console.WriteLine("Data:");
                Console.WriteLine("{0,-5} | {1}", "Author", "Name");
                Console.WriteLine(new string('-', 50));
                foreach (var item in firstQueryResults)
                    Console.WriteLine("{0,-5} | {1} ", item.Author, item.Name);

                // Write an analogue of the SQL query:
                //   SELECT author, name FROM books
                //   WHERE to_tsvector(content) @@ to_tsquery('king & palace & !Norfolk')
                var secondQuery = ctx.Books
                    .Where(b => PgSqlTextFunctions.Match(
                        PgSqlTextFunctions.ToTsVector(b.Content), 
                        PgSqlTextFunctions.ToTsQuery("king & palace & !Norfolk"))
                    )
                    .Select(b => new { b.Author, b.Name });
                var secondQueryResults = secondQuery.ToList();

                // Output:
                Console.WriteLine();
                Console.WriteLine("2) Boolean mode");
                Console.WriteLine();
                Console.WriteLine("SQL:");
                Console.WriteLine(secondQuery.ToString());
                Console.WriteLine();
                Console.WriteLine("Data:");
                Console.WriteLine("{0,-5} | {1}", "Author", "Name");
                Console.WriteLine(new string('-', 50));
                foreach (var item in secondQueryResults)
                    Console.WriteLine("{0,-5} | {1} ", item.Author, item.Name);

                Console.ReadLine();

            }
        }
    }
}